/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvSignalVar.h"
#include "HvControlBinop.h"
#include "HvMath.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_jF0MXgCI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ALdc7SVA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_h58E7ULY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SHbKgPlh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_TVjFbXAd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_l9bMGW36_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xtMdlEfy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_6ZfrSloA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PAAfU2lH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_k8W5xKjZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_b6yCOHc4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_y28B7zUJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_5jUtAMEz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_yCkvNiTU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xq7lOW6U_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_6HdpHWnT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_6UnMo5am_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ViIO2mQr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5BJV8twW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_KlV2gYgw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IkMCYiZL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NpzW8iYn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_pCmKTaqm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_afvOB1Cg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_wkDRBQCm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_TDdNGgbq_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_lQMgmvUo;
  SignalLine sLine_rylvXKbd;
  SignalLine sLine_9j1m7DM8;
  ControlBinop cBinop_jF0MXgCI;
  ControlBinop cBinop_ALdc7SVA;
  ControlBinop cBinop_h58E7ULY;
  ControlBinop cBinop_SHbKgPlh;
  ControlBinop cBinop_xtMdlEfy;
  ControlBinop cBinop_PAAfU2lH;
  ControlBinop cBinop_k8W5xKjZ;
  ControlBinop cBinop_b6yCOHc4;
  ControlVar cVar_y28B7zUJ;
  ControlVar cVar_5jUtAMEz;
  SignalVarf sVarf_40iiypM5;
  SignalVarf sVarf_9r4glEHB;
  SignalVarf sVarf_eK7PCYNH;
  SignalVarf sVarf_VaO0T4iF;
  ControlBinop cBinop_ViIO2mQr;
  ControlBinop cBinop_NpzW8iYn;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
